import selenium.webdriver

from automon.log import Logging

from .config_webdriver import ConfigWebdriver

log = Logging(name='SeleniumConfig', level=Logging.INFO)


class SeleniumConfig(object):
    webdriver_wrapper: ConfigWebdriver

    def __init__(self):
        self._webdriver_wrapper = ConfigWebdriver()

    def __repr__(self):
        return f'{self.driver}'

    @property
    def driver(self):
        return self.webdriver

    @property
    def set_webdriver(self):
        return self._webdriver_wrapper

    @property
    def webdriver(self):
        if self.set_webdriver.webdriver:
            return self.set_webdriver.webdriver
        else:
            log.debug('waiting for driver')



import datetime
import logging

import pandas as pd
import numpy as np
import tracemalloc

import unittest

import automon
from automon.integrations.google.sheets import GoogleSheetsClient
from automon.integrations.facebook import FacebookGroups
from automon import Logging

logging.getLogger('SeleniumBrowser').setLevel(logging.CRITICAL)
logging.getLogger('FacebookGroups').setLevel(logging.CRITICAL)
logging.getLogger('ConfigChrome').setLevel(logging.ERROR)
logging.getLogger('RequestsClient').setLevel(logging.INFO)

tracemalloc.start()

log = Logging(level=Logging.INFO)


def get_facebook_info(url: str):
    if not url:
        return {}

    group = FacebookGroups(
        url=url_cleaner(url=url)
    )
    # group.start(headless=False)
    group.start(headless=True)
    group.get_about()
    # if not group.privacy_details:
    #     close = group._browser.wait_for(group.xpath_popup_close)
    #     group._browser.action_click(close)
    #     about = group._browser.wait_for(group.xpath_about)
    #     group._browser.action_click(about)

    return group.to_dict()


def url_cleaner(url: str):
    if url[-1] == '/':
        url = url[:-1]
    return url


class MyTestCase(unittest.TestCase):
    def test_authenticate(self):
        spreadsheetId = '1isrvjU0DaRijEztByQuT9u40TaCOCwdaLAXgGmKHap8'
        test = GoogleSheetsClient(
            spreadsheetId=spreadsheetId,
            worksheet='Automated Count WIP',
        )

        if not test.authenticate():
            return

        def merge_urls():
            test.get(
                ranges='AUDIT list Shelley!A:Z',
                fields="sheets/data/rowData/values/hyperlink",
            )

            data = test.response['sheets'][0]['data'][0]['rowData']
            # expand nested data
            links = []
            for x in data:
                if x:
                    links.append(
                        x['values'][0]['hyperlink']
                    )

            df_Shelley = pd.DataFrame(data=links, columns=['url'])

            test.get()
            test.get_values(
                range='Automated Count WIP!A:Z'
            )

            sheet_values = test.values
            sheet_columns = sheet_values[0]
            sheet_data = sheet_values[1:]

            df = pd.DataFrame(data=sheet_data, columns=sheet_columns)
            df = df.dropna(subset=['url'])

            # merge both lists or urls
            df = pd.merge(df, df_Shelley, how='outer', on='url')
            df = df.drop_duplicates(subset=['url'], keep='first')
            return df

        def batch_processing(index: int, df: pd.DataFrame):
            df_results = df['url'].dropna().apply(
                lambda url: get_facebook_info(url=url)
            )
            df_results = pd.DataFrame(df_results.tolist())

            todays_date = datetime.datetime.now().date()

            df = df.reset_index()
            df = df.drop('index', axis=1)

            # create columns
            df[f'url'] = df_results['url']
            df[f'{todays_date}'] = df_results['members_count']
            df[f'title'] = df_results['title']
            df[f'content_unavailable'] = df_results['content_unavailable']
            df[f'creation_date'] = df_results['creation_date']
            df[f'creation_date_timestamp'] = df_results['creation_date_timestamp']
            df[f'history'] = df_results['history']
            df[f'members_count'] = df_results['members_count']
            df[f'posts_monthly_count'] = df_results['posts_monthly_count']
            df[f'posts_today_count'] = df_results['posts_today_count']
            df[f'privacy'] = df_results['privacy']
            df[f'visible'] = df_results['visible']

            # set dtype to Int32
            df[f'{todays_date}'] = df[f'{todays_date}'].astype('Int32')
            df[f'creation_date_timestamp'] = df[f'creation_date_timestamp'].astype('Int32')
            df[f'members_count'] = df[f'members_count'].astype('Int32')
            df[f'posts_monthly_count'] = df[f'posts_monthly_count'].astype('Int32')
            df[f'posts_today_count'] = df[f'posts_today_count'].astype('Int32')

            # order columns
            columns = [
                'url',
                'title',
                'creation_date',
                'creation_date_timestamp',
                'history',
                'privacy',
                'visible',
                'content_unavailable',
                'posts_monthly_count',
                'posts_today_count',
                'members_count',
            ]

            # add all other dates
            df_columns = df.columns.tolist()
            columns.extend(
                [x for x in df_columns if x not in columns]
            )

            # finally add today's date
            if f'{todays_date}' not in columns:
                columns.append(
                    f'{todays_date}',
                )

            df = df.loc[:, columns]
            df = df.fillna(np.nan).replace([np.nan], [None])

            sheet_index = index + 2

            update_columns = test.update(
                range=f'Automated Count WIP!A1:Z',
                values=[columns],
            )

            update = test.update(
                range=f'Automated Count WIP!A{sheet_index}:Z',
                values=[x for x in df.values.tolist()]
            )

            log.info(
                f'{[x for x in df.values.tolist()]}'
            )

            return df

        def memory_profiler():
            snapshot = tracemalloc.take_snapshot()
            top_stats = snapshot.statistics("lineno")

            df_memory_profile = pd.DataFrame([
                dict(size_B=stat.size, count=stat.count, file=stat.traceback._frames[0][0],
                     file_line=stat.traceback._frames[0][1]) for stat in top_stats
            ])
            df_memory_profile.sort_values(by='size_B', ascending=False)
            df_memory_profile['size_KB'] = df_memory_profile['size_B'].apply(
                lambda B: round(B / 1024)
            )
            df_memory_profile['size_MB'] = df_memory_profile['size_KB'].apply(
                lambda KB: round(KB / 1024)
            )
            cols = df_memory_profile.columns.tolist()
            cols.sort()
            df_memory_profile = df_memory_profile.loc[:, cols]

            log.debug(
                f"total memory used: {df_memory_profile['size_MB'].sum()} MB; "
                f'most memory used: '
                f'{df_memory_profile.iloc[0].to_dict()}'
            )

            return df_memory_profile

        # start processing
        test.get_values(
            range='Automated Count WIP!A:Z'
        )

        sheet_values = test.values
        sheet_columns = sheet_values[0]
        sheet_data = sheet_values[1:]

        df = pd.DataFrame(data=sheet_data, columns=sheet_columns)
        df = df.dropna(subset=['url'])

        BATCH_SIZE = 1

        i = 0 + 6146
        i = 0
        while i < len(df):
            df_batch = df[i:i + BATCH_SIZE]
            try:
                df_batch = batch_processing(index=i, df=df_batch)
                df_memory = memory_profiler()
            except Exception as e:
                df_memory = memory_profiler()
                pass
            i += 1

            pass

        pass


if __name__ == '__main__':
    unittest.main()



import selenium.webdriver

from automon.log import Logging

from .config_webdriver_chrome import ConfigChrome

log = Logging(name='ConfigWebdriver', level=Logging.INFO)


class ConfigWebdriver(object):
    webdriver: selenium.webdriver
    webdriver_wrapper: any

    Chrome: ConfigChrome
    Edge: NotImplemented
    Firefox: NotImplemented

    def __init__(self):
        self._webdriver_wrapper = None

        self._chrome = ConfigChrome()
        self._edge = NotImplemented
        self._firefox = NotImplemented

    def __repr__(self):
        if self._webdriver_wrapper:
            return f'{self._webdriver_wrapper}'
        return f'webdriver not configured. try selecting a webdriver'

    @property
    def driver(self):
        """alias to webdriver

        """
        return self.webdriver

    @property
    def webdriver(self):
        """selenium webdriver

        """
        return self.webdriver_wrapper.webdriver

    @property
    def webdriver_wrapper(self) -> any or ConfigChrome:
        """webdriver wrapper

        """
        return self._webdriver_wrapper

    @property
    def window_size(self):
        """get window size

        """
        if self.webdriver_wrapper:
            return self.webdriver_wrapper.window_size

    def Chrome(self):
        """selenium Chrome webdriver

        """
        self._webdriver_wrapper = self._chrome
        return self._webdriver_wrapper

    def Edge(self):
        """selenium Edge webdriver

        """
        return self._edge

    def Firefox(self):
        """selenium Firefox webdriver

        """
        return self._firefox

    def run(self):
        """run webdriver"""
        try:
            return self.webdriver_wrapper.run()
        except Exception as e:
            log.error(f'failed to run: {e}', enable_traceback=False)

    def start(self):
        """alias to run"""
        return self.run()

    def quit(self):
        """quit webdriver"""
        return



import os
import tempfile
import functools
import selenium
import selenium.webdriver

from selenium.webdriver.common.by import By
from selenium.webdriver.common.keys import Keys
from urllib.parse import urlparse
from bs4 import BeautifulSoup

from automon.log import Logging
from automon.helpers.dates import Dates
from automon.helpers.sleeper import Sleeper
from automon.helpers.sanitation import Sanitation
from automon.integrations.requestsWrapper import RequestsClient

from .config import SeleniumConfig
from .browser_types import SeleniumBrowserType
from .user_agents import SeleniumUserAgentBuilder

log = Logging(name='SeleniumBrowser', level=Logging.DEBUG)


class SeleniumBrowser(object):
    config: SeleniumConfig
    webdriver: selenium.webdriver
    status: int

    def __init__(self, config: SeleniumConfig = None):
        """A selenium wrapper"""

        self._config = config or SeleniumConfig()
        self.request = None

    def __repr__(self):
        if self.url:
            return f'{self.webdriver.name} {self.status} {self.webdriver.current_url} {self.window_size}'
        return f'{self.webdriver}'

    @property
    def by(self) -> By:
        """Set of supported locator strategies"""
        return selenium.webdriver.common.by.By()

    @property
    def config(self):
        return self._config

    @property
    def webdriver(self):
        return self.config.webdriver

    @property
    def get_log(self) -> list:
        """Gets the log for a given log type"""
        logs = []
        for log_type in self.webdriver.log_types:
            logs.append(
                {
                    log_type: self.webdriver.get_log(log_type)
                }
            )

        return logs

    @property
    def keys(self):
        """Set of special keys codes"""
        return selenium.webdriver.common.keys.Keys

    @property
    def status(self):
        if self.request is not None:
            return self.request.results.status_code

    # @property
    # def type(self) -> SeleniumBrowserType:
    #     return SeleniumBrowserType(self.config)

    @property
    def url(self):
        if self.webdriver.current_url == 'data:,':
            return ''
        return self.webdriver.current_url

    @property
    def window_size(self):
        return self.config.set_webdriver.window_size

    def _is_running(func) -> functools.wraps:
        @functools.wraps(func)
        def wrapped(self, *args, **kwargs):
            if self.webdriver is not None:
                return func(self, *args, **kwargs)
            return False

        return wrapped

    def _screenshot_name(self, prefix=None):
        """Generate a unique filename"""

        title = self.webdriver.title
        url = self.webdriver.current_url
        hostname = urlparse(url).hostname

        hostname_ = Sanitation.ascii_numeric_only(hostname)
        title_ = Sanitation.ascii_numeric_only(title)
        timestamp = Dates.filename_timestamp()

        if prefix:
            prefix = Sanitation.safe_string(prefix)
            return f'{prefix}_{hostname_}_{title_}_{timestamp}.png'

        return f'{hostname_}_{title_}_{timestamp}.png'

    @_is_running
    def action_click(self, xpath: str, note: str = None) -> str or False:
        """perform mouse command"""
        try:
            click = self.find_element(value=xpath, by=self.by.XPATH)
            click.click()
            if note:
                log.debug(f'click: ({note}) {xpath}')
            else:
                log.debug(f'click: {xpath}')
            return click
        except Exception as e:
            log.error(f'failed to click: {xpath}, {e}', enable_traceback=False)
        return False

    @_is_running
    def action_type(self, key: str or Keys, secret: bool = False):
        """perform keyboard command"""
        try:
            actions = selenium.webdriver.common.action_chains.ActionChains(
                self.webdriver)
            actions.send_keys(key)
            actions.perform()

            if secret:
                key = f'*' * len(key)

            log.debug(f'type: {key}')
            return True
        except Exception as e:
            log.error(f'failed to type: {key}, {e}', enable_traceback=False)
        return False

    @_is_running
    def close(self):
        """close browser"""
        log.info(f'Browser closed')
        self.webdriver.close()

    @_is_running
    def find_element(
            self,
            value: str,
            by: By.ID = By.ID,
            **kwargs):
        """find element"""
        element = self.webdriver.find_element(value=value, by=by, **kwargs)
        log.debug(f'found element: {self.url} {element.text}')
        return element

    @_is_running
    def find_xpath(self, value: str, by: By = By.XPATH, **kwargs):
        """find xpath"""
        xpath = self.find_element(value=value, by=by, **kwargs)
        log.debug(f'found xpath: {self.url} {xpath.text}')
        return xpath

    @_is_running
    def get(self, url: str, **kwargs) -> bool:
        """get url"""
        try:
            self.webdriver.get(url, **kwargs)
            self.request = RequestsClient(url=url)

            msg = f'GET {self.status} {self.webdriver.current_url}'
            if kwargs:
                msg += f', {kwargs}'
            log.debug(msg)
            return True
        except Exception as e:
            self.request = RequestsClient(url=url)
            msg = f'GET {self.status}: {e}'
            log.error(msg, enable_traceback=False)

        return False

    @_is_running
    def get_page(self, *args, **kwargs):
        """alias to get"""
        return self.get(*args, **kwargs)

    @_is_running
    def get_page_source(self) -> str:
        """get page source"""
        return self.webdriver.page_source

    @_is_running
    def get_page_source_beautifulsoup(self, markdup: str = None, features: str = 'lxml') -> BeautifulSoup:
        """read page source with beautifulsoup"""
        if not markdup:
            markdup = self.get_page_source()
        return BeautifulSoup(markup=markdup, features=features)

    def get_random_user_agent(self, filter: list or str = None, case_sensitive: bool = False) -> list:
        return SeleniumUserAgentBuilder().get_random(filter=filter, case_sensitive=case_sensitive)

    @_is_running
    def get_screenshot_as_png(self, **kwargs):
        """screenshot as png"""
        return self.webdriver.get_screenshot_as_png(**kwargs)

    @_is_running
    def get_screenshot_as_base64(self, **kwargs):
        """screenshot as base64"""
        return self.webdriver.get_screenshot_as_base64(**kwargs)

    @_is_running
    def get_user_agent(self):
        return self.webdriver.execute_script("return navigator.userAgent")

    @_is_running
    def is_running(self) -> bool:
        """browser is running"""
        return True

    @_is_running
    def quit(self) -> bool:
        """gracefully quit browser"""
        try:
            self.webdriver.close()
            self.webdriver.quit()
            self.webdriver.stop_client()
        except Exception as error:
            log.error(f'failed to quit browser. {error}')
            return False
        return True

    @_is_running
    def save_screenshot(
            self,
            filename: str = None,
            prefix: str = None,
            folder: str = None,
            **kwargs) -> bool:
        """save screenshot to file"""

        if not filename:
            filename = self._screenshot_name(prefix)

        if not folder:
            path = os.path.abspath(tempfile.gettempdir())
        else:
            path = os.path.abspath(folder)

        if not os.path.exists(path):
            os.makedirs(path)

        save = os.path.join(path, filename)

        log.info(f'Saving screenshot to: {save}')

        return self.webdriver.save_screenshot(save, **kwargs)

    @_is_running
    def set_window_size(self, width=1920, height=1080, device_type=None) -> bool:
        """set browser resolution"""

        try:
            self.config.set_webdriver.webdriver_wrapper.set_window_size(width=width, height=height,
                                                                        device_type=device_type)
        except Exception as error:
            log.error(f'failed to set resolution. {error}')
            return False
        return True

    def run(self):
        """run browser"""
        return self.config.set_webdriver.run()

    def start(self):
        """alias to run"""
        return self.run()

    def wait_for(
            self,
            value: str or list,
            by: By = By.XPATH,
            retries: int = 3,
            **kwargs) -> str or False:
        """wait for something"""
        retry = 1
        while True:
            try:
                if isinstance(value, list):
                    values = value
                    for value in values:
                        try:
                            self.find_element(
                                by=by,
                                value=value,
                                **kwargs)
                            log.debug(f'waiting for {by}: {self.url} {value}')
                            return value
                        except:
                            log.error(f'{by} not found: {self.url} {value}', enable_traceback=False)
                else:
                    self.find_element(
                        by=by,
                        value=value,
                        **kwargs)
                    log.debug(f'waiting for {by}: {self.url} {value}')
                    return value
            except Exception as error:
                log.error(f'not found {by}: {self.url} {value}, {error}',
                          enable_traceback=False)
                Sleeper.seconds(f'wait for', 1)

            retry += 1

            if retry > retries:
                log.error(f'max wait reached: {self.url}', enable_traceback=False)
                break
        return False

    def wait_for_element(self, element: str or list, **kwargs) -> str or False:
        """wait for an element"""
        return self.wait_for(value=element, by=self.by.ID, **kwargs)

    def wait_for_xpath(self, xpath: str or list, **kwargs) -> str or False:
        """wait for an xpath"""
        return self.wait_for(value=xpath, by=self.by.XPATH, **kwargs)
